# Copyright IBM Corporation 2025, 2026
# SPDX-License-Identifier: MIT

from orchestrator.modules.module import ModuleTypeEnum

module_types_provided = [ModuleTypeEnum.ACTUATOR, ModuleTypeEnum.SAMPLE_STORE]
